#!/usr/bin/ruby

# EliseWebServer
# Fichier : Content.rb
# version : 3
# Date cration : 21.06.04
# Date modification : 9.02.05
#
# Reprsente le contenu d'une URL.

require 'thread'
require 'stringio'

require 'src/DirList.rb'
require 'src/HTTPTime.rb'
require 'src/Session.rb'
require 'src/Cache.rb'

require 'src/constants.rb'
require 'user_settings.rb'

# Permet de crer un contenu  partir d'une URL.
class Content
   # Renvoie une instance d'une des classes suivante :
   # - 'ContentRedirection' : redirection
   # - 'ContentFile' : le contenu d'un fichier
   # - 'ContentDir' : le listage des fichiers d'un repertoire
   # - 'ContentDynamic' : interprtation d'un fichier
   # - 'ContentApplication' : appel  une application charge en mmoire
   # - 'ContentError' : une erreur HTTP
   #
   # messageClient [InHTTPMessage] : les donnes du client
   # ret [ContentCommon] : un contenu
   def Content::new(messageClient)
      if File::directory?(messageClient.getAbsoluteFilename)
         #si il n'y a pas de /  la fin alors envois une redirection au client
         if (/^.*\/$/ =~ messageClient.getAbsoluteFilename) == nil
             return ContentRedirection::new(messageClient.getRelativeFilename + '/')
         end

         #test si le rpertoire contient un index par dfaut
         #si oui alors renvoie son contenu sinon renvoie la liste des fichiers
         index = Dir::entries(messageClient.getAbsoluteFilename) & INDEXES
         if index.empty?
            return ContentDir::new(messageClient.getAbsoluteFilename)
         else
            messageClient.setAbsoluteFilename(messageClient.getAbsoluteFilename + index[0])
            return Content::contentFile(messageClient)
         end
      end

      return Content::contentFile(messageClient)
   end

   # Renvoie un contenu de fichier.
   #
   # messageClient [InHTTPMessage] : les donnes du client
   # ret [ContentCommon] : un contenu
   def Content::contentFile(messageClient)

      #test si le fichier existe, si non alors renvois une erreur
      if !messageClient.ressourceExiste?
         #!File::exist?(messageClient.getAbsoluteFilename)
         return ContentError::new(404)
      end

      #change le repertoire courant (pour les scripts dynamiques)
      Dir::chdir(messageClient.getAbsoluteDirectory)

      #si c'est une application (la liste des applications se trouve dans 'user_settings.rb')
      if nil != app = APPLICATIONS[messageClient.getRelativeFilename]
         return ContentApplication::new(messageClient, app)
      end

      #test si c'est un contenu dynamique
      /^.*\.(.+)$/ =~ messageClient.getAbsoluteFilename
      if DYNAMIC_FILES.include?($1)
         return ContentDynamic::new(messageClient)
      #sinon renvois le fichier tel quel
      else
         return ContentFile::new(messageClient)
      end
   end

end

# Classe de base pour tout les types de contenu.
class ContentCommon
   @keepAlive #si la connexion est persistante
   @nbMess #le nombre de message pouvant encore tre accept sur la mme connextion (permet de l'indiquer au client)

   def initialize
      @keepAlive = true
   end

   # Ecrit le contenu sur le socket
   #
   # con [IO] : le socket
   # state_code [Integer] : le code d'tat du message (voir 'constant.rb')
   def put(con, state_code=200)
      con.write(
         "HTTP/1.1 #{state_code.to_s} #{STATE_CODE[state_code]}" + $crlf +
         "Date: #{HTTPTime::new.getHTTP}" + $crlf +
         'Server: ' + SERVER_TAG + $crlf
      )
      con.write('Connection: Keep-Alive' + $crlf) if @continue
      con.write("Keep-Alive: timeout=#{MAX_CONNEXION_WAIT}, max=#{MAX_NUMBER_OF_HTTP_MESSAGE_ON_THE_SAME_CONNEXION-@nbMess}" + $crlf) if @nbMess != nil
      con.write('Content-Length: ' + self.size.to_s + $crlf)
      con.write('Content-Type: ' + self.typeMIME + $crlf)
   end

   # Renvoie le type MIME, si cette mthode n'est pas dfinit
   # dans la classe enfant alors renvoi le type par dfaut.
   #
   # ret [String] : le type MIME (par exemple 'text/hml')
   def typeMIME
      TYPE_MIME[MIME_DEFAULT]
   end

   # Dfinit le nombre de message pouvant encore tre transmit sur la connexion TCP courante.
   #
   #nbMess [Integer] : le nombre de message
   def setNbMess(nbMess)
      @nbMess = nbMess
   end

   # Renvoie la taille du contenu.
   #
   # ret [Integer] : la taille du contenu
   def size
      0
   end

   # Renvois la date de la dernire modification du contenu.
   # Par dfaut renvois la date courante.
   #
   # ret [HTTPTime]
   def getLastModifiedDate
      HTTPTime::new
   end

   # Est-ce que la connexion doit tre maintenu ?
   # (pour pouvoir envoyer/recevoir plusieurs messages sur la mme connexion).
   #
   # ret [Boolean] : si la connexion peut tre maintenue
   def keepAlive?
      return @keepAlive
   end

   # Dfinit si la connexion doit tre maintenu.
   # (pour pouvoir envoyer/recevoir plusieurs messages sur la mme connexion).
   #
   # keepAlice [Boolean] : si la connexion peut tre maintenue
   def keepAlive(keepAlive)
      @keepAlive = keepAlive
   end
end

# Reprsente un fichier sur le disque
class ContentFile < ContentCommon
   @messageClient #le chemin absolu du fichier
   @content #le contenu
   @lastModifiedDate
   @size

   # file : le chemin absolu du fichier
   def initialize(messageClient)
      super()
      @messageClient = messageClient

      begin
         #lit le fichier dans le cache
         @content = Cache::readFile(@messageClient.getAbsoluteFilename, true)
         @lastModifiedDate = HTTPTime::fromTime(@content[0])
         @size = @content[1].size
      rescue FileToBigException
         @lastModifiedDate = HTTPTime::fromTime(File::mtime(@messageClient.getAbsoluteFilename))
         @size = File::size(@messageClient.getAbsoluteFilename)
      end
   end

   # Renvoie la taille du contenu.
   #
   # ret : la taille du contenu
   def size
      return @size
   end

   # Renvois la date de la dernire modification du contenu.
   # Par dfaut renvois la date courante.
   #
   # ret [HTTPTime]
   def getLastModifiedDate
      @lastModifiedDate
   end

   # Renvoie le type MIME.
   #
   # ret [String] : le type MIME
   def typeMIME
      /^.*\.(.+)$/ =~ @messageClient.getAbsoluteFilename #capture l'extension
      if TYPE_MIME.key?($1) then TYPE_MIME[$1] else TYPE_MIME[MIME_DEFAULT] end
   end

   # Ecrit le contenu sur le socket.
   #
   # con [IO] : le socket
   def put(con)
      super
      con.write("Last-Modified: #{@lastModifiedDate.getHTTP}" + $crlf)
      con.write($crlf)

      if @content != nil
         con.write(@content[1])
      else
         file = Cache::openFile(@messageClient.getAbsoluteFilename)
         begin
            size_sended = 0
            while size_sended < @size 
               con.write(file.read(BIN_FILE_BUFFER))
               size_sended += BIN_FILE_BUFFER
            end
         rescue Exception
         end
      end
   end
   
end

# Reprsente le contenu d'un rpertoire
class ContentDir < ContentCommon
   @content

   # dir : le rpertoire  lister (chemin absolu)
   def initialize(dir)
      super()
      list = DirList::new(dir)
      @content = SERVER_PAGE.sub('{content}', list.print)
   end

   # Renvoie la taille du contenu.
   #
   # ret [Integer]
   def size
      @content.size
   end

   # Ecrit le contenu sur le socket.
   #
   # con [IO] : le socket
   def put(con)
      super
      con.write($crlf)

      con.write(@content)
   end
end

# Reprsente un contenu dynamique, c--d un
# fichier ruby interpret  la vole.
class ContentDynamic < ContentCommon
   @messageClient
   @size
   def initialize(messageClient)
      super()
      @messageClient = messageClient
   end

   def size
      return @size
   end

   def put(con)
      out = StringIO::new

      session = Session::new(@messageClient.getOption('Cookie'))

      Thread::new{
         begin
            m = Module::new
            m.const_set("OUT", out)
            m.const_set("SESSION", session)
            m.const_set("INPUT", @messageClient.getInput)
            m.module_eval(
               "$SAFE = 0;\n" + Cache::readFile(@messageClient.getAbsoluteFilename)[1]
            )
         rescue  Exception => e
            out.write(SERVER_PAGE.sub('{content}', e.message))
            puts e.backtrace
         end
      }.join

      result_content = out.string
      @size = result_content.size

      super
      if session.createCookie?
         expires = session.getDateExpire
         con.write("Set-Cookie: #{COOKIE_SESSION_NAME}=" + session.getID.to_s + ( expires == nil ? '' : '; expires='+expires ) + $crlf);
      end
      con.write($crlf)
      con.write(result_content)
   end
end

# Reprsente le contenu d'une application.
# Les applications sont charge au lancement du serveur.
# La liste des applications est dfinit dans 'user_settings.rb'
class ContentApplication < ContentCommon
   @messageClient
   @app
   @size = 0
   def initialize(messageClient, app)
      super()
      @messageClient = messageClient
      @app = app
   end

   def size
      return @size
   end

   def put(con)
      out = StringIO::new

      session = Session::new(@messageClient.getOption('Cookie'))
      
      #(re)charge l'application si besoin
      require @messageClient.getFile

      #l'application peut retourner un code d'erreur (par exemple redirection : 301)
      code = nil
      Thread::new{
         begin
            m = Module::new
            m.const_set("OUT", out)
            m.const_set("SESSION", session)
            m.const_set("INPUT", @messageClient.getInput)
            m.module_eval(
               "$SAFE = 0;\n ;code = " + @app.id2name + "::entryPoint(const_get(:OUT), const_get(:INPUT), const_get(:SESSION))"
            )
         rescue  Exception => e
            puts e.message
         end
      }.join
     
      #code de redirection
      case code
      when 301
         ContentRedirection::new(@messageClient.getRelativeFilename).put(con)
         return
      else
         if code != nil
            ContentError::new(code).put(con)
            return
         end
      end
         
      result_content = out.string
      @size = result_content.size

      super
      if session.createCookie?
         expires = session.getDateExpire
         con.write("Set-Cookie: #{COOKIE_SESSION_NAME}=" + session.getID.to_s + ( expires == nil ? '' : '; expires='+expires ) + $crlf);
      end
      con.write($crlf)
      con.write(result_content)
   end
end

# Reprsente une redirection (301)
class ContentRedirection < ContentCommon
   @file #le chemin vers le nouveau fichier

   # file : le chemin vers le nouveau fichier
   def initialize(file)
      super()
      @file = file
   end

   # Ecrit le contenu sur le socket.
   #
   # con [IO] : le socket
   def put(con)
      super(con, 301)
      con.write("Location: " + @file + $crlf)
      con.write($crlf)
   end
end

# Reprsente une erreur (par exemple 404)
class ContentError < ContentCommon
   @error_code
   def initialize(error_code)
      super()
      @error_code = error_code
   end

   def size
      return @size
   end

   def put(con)
      content = $crlf + SERVER_PAGE.sub('{content}', @error_code.to_s + ' ' + STATE_CODE[@error_code])
      @size = content.size

      super(con, @error_code)

      con.write($crlf)
      con.write(content)
   end
end

# Reprsente un code gnrique
class ContentGenericCode < ContentCommon
   @statCode
   def initialize(statCode)
      super()
      @statCode = statCode
   end

   def put(con)
      super(con, @statCode)

      con.write($crlf)
   end
end
