#!/usr/bin/ruby

# EliseWebServer
# Fichier : Cache.rb
# version : 1
# Date cration : 8.02.05
# Date modification : 9.02.05
#
# Gestion du cache, lors du chargement d'un
# fichier sur le disque sa date est compare  celle
# du cache, si elle est la mme alors le fichier
# n'est pas recharg est pris directement dans le cache.

require 'user_settings.rb'

# Exception leve lorsque un fichier ne peut pas tenir completement
# en mmoire.
class FileToBigException < Exception
end

class Cache
   @@actualSize = 0 #la taille total du cache  un certain moment
   #les fichiers en cache :
   #{ <path> => [ <date de la dernire modification de la donne en cache>, <date de la dernire modification du fichier>, <donne> ] }
   @@files = Hash::new

   #permet de protger l'accs au cache pour les accs concurrents
   @@mutex = Mutex::new

   #le temps de vie d'une donne en cache [s]
   #pass ce temps la donne est efface du cache
   #(un thread s'occupe de vrifier l'tat du cache priodiquement)
   @@LifeOfCachedData = 60 * 60 * 24 #(1 jour)

   #l'intervale de temps entre les contrles des donnes du cache [s]
   @@CheckInterval = 10 * 60 #(10 minutes)

   #thread s'occupant de vider les entres du cache trop vieilles
   Thread::new{
      begin
      while true
         sleep @@CheckInterval
         message = "[Cache] Vrification du cache, taille avant : #{@@actualSize} o"
         #limite les accs concurrents sur le cache
         @@mutex.synchronize{
            @@files.delete_if{|path, content|
               c = content[0] + @@LifeOfCachedData < Time::new
               @@actualSize -= content[2].size if c
               c
            }
         }
         message += ", taille aprs : #{@@actualSize} o"
         puts message
      end
      rescue Exception => e
            puts e.message
            puts e.backtrace
      end
   }


   # Renvoie le contenu d'un fichier.
   # Si checkSize est  'true' alors lve une erreur (FileToBigException) si la taille du fichier excde
   # la constante 'CACHE_MAX_SIZE_FILE'.
   # L'existance de fichier 'file' sur le dique n'est pas vrifie.
   #
   # ret [ <date de modification sur le disque du fichier>, <donnes du fichier> ] : Les donnes du fichier
   # file : le chemin complet absolu vers le fichier : 
   # checkSize : vrifie si le fichier peut tenir en cache, si non alors raise FileToBigException
   def Cache::readFile(file, checkSize = false)

      #taille du fichier
      size = File::size(file)

      #si il excde la taille maximal autorise d'un fichier cach
      if size / 1048576 > CACHE_MAX_SIZE_FILE
         raise FileToBigException if checkSize

         return [File::mtime(file), Cache::openFile(file).read]
      end
      
      #attention aux accs concurrents au cache
      @@mutex.synchronize{

         content =  @@files[file]
         #le fichier n'existe pas en cache
         if content == nil
            f = Cache::openFile(file)
            
            content = [Time::new, File::mtime(file), f.read]
            if (@@actualSize + size) / 1048576 < CACHE_SIZE
               @@files[file] = content
               @@actualSize += content[2].size
            end
            f.close
            return content[1..2]

         #si la date du fichier est la mme que celle en cache alors renvoie le cache
         elsif File::mtime(file) == content[1]
            return content[1..2]

         #sinon recharge le fichier
         else
            @@actualSize -= content[2].size
            f = Cache::openFile(file)
            content = [Time::new, File::mtime(file), f.read]
            if (@@actualSize + content[2].size) / 1048576 > CACHE_SIZE
               @@files.delete(file)
            else
               @@files[file] = content
               @@actualSize += content[2].size
            end
            f.close
            return content[1..2]
         end
      }
   end
   
   def Cache::openFile(file, mode='r')
      if /.*mswin32.*/ =~ RUBY_PLATFORM
         return File::new(file, mode + 'b')
      else
         return File::new(file, mode)
      end
   end   
end
