/**
  * Script de cration de la base de donne FiberNet.
  * Auteur  : G.Burri
  * Date    : 27.01.05
  * Version : 1
  */

/**
  * Table de base de tous les objets fibernet.
  */
CREATE TABLE ObjetFibernet (
  -- la clef primaire
  idObjet serial NOT NULL,
  
  PRIMARY KEY(idObjet)
);

/**
  * Ensemble d'attribut additionnel appartenant 
  */  
CREATE TABLE GroupeAttributAdditionnel (
  id serial NOT NULL,
  
  PRIMARY KEY(id)
);

/**
  * Table de base des objets pouvant possder des attributs supplmentaires.
  */
CREATE TABLE ObjetReseau (
  idReseau serial NOT NULL,
  groupeAttributAdditionnel integer NULL,
  
  observations text NOT NULL DEFAULT '',

  PRIMARY KEY(idObjet)
) INHERITS(ObjetFibernet);
CREATE INDEX ObjetReseau_groupeAttributAdditionnel_INDEX on ObjetReseau(groupeAttributAdditionnel);

/**
  * La dfinition d'un attribut additionnel.
  */
CREATE TABLE AttributDef (
  -- la clef primaire
  id serial NOT NULL,

  nom varchar(45),
  typeAttribut varchar(20),

  PRIMARY KEY(id)
);

/**
  * Attribut additionnel, chaque objet
  * fibernet (voir tyble "ObjetReseau" peut possder un ou plusieurs
  * attributs additionnels.  
  */  
CREATE TABLE AttributAdditionnel (
  -- l'identificateur
  id serial,
  -- clef trangre vers un groupe d'attribut
  groupeAttributAdditionnel integer NOT NULL REFERENCES GroupeAttributAdditionnel ON DELETE CASCADE,
  -- clef trangre vers un AttributDef
  attributDef integer NOT NULL REFERENCES AttributDef ON DELETE CASCADE,
  
  PRIMARY KEY(id)
);
CREATE INDEX AttributAdditionnel_groupeAttributAdditionnel_INDEX on AttributAdditionnel(GroupeAttributAdditionnel);
CREATE INDEX AttributAdditionnel_attributDef_INDEX on AttributAdditionnel(AttributDef);

/**
  * Attribut additionnel de type float.
  */
CREATE TABLE AAfloat (  
  value integer NULL
) INHERITS (AttributAdditionnel);

/**
  * Attribut additionnel de type entier.
  */
CREATE TABLE AAint (
  value integer NULL
) INHERITS (AttributAdditionnel);

/**
  * Attribut additionnel de type chaine de caractre.
  */
CREATE TABLE AAstring (
  value varchar(255) NULL
) INHERITS (AttributAdditionnel);

/**
  * Classe gnrale pour dcrire un type.
  */
CREATE TABLE "Type" (
   id serial NOT NULL,

   nom varchar(100) UNIQUE NOT NULL,
   
   PRIMARY KEY(id)
) INHERITS(ObjetFibernet);

/**
  * Le type de connecteur (utiliser pour les bretelles
  * et les adaptateurs se montant sur les racks)
  */
CREATE TABLE TypeConnecteur (
  idTypeConnecteur serial NOT NULL,

  PRIMARY KEY(idTypeConnecteur)
) INHERITS("Type");

/**
  * Personne, reprsente une personne physique ou morale.
  */
CREATE TABLE Personne (
  id serial NOT NULL,

  nom varchar(100) NOT NULL,
  identifiant varchar(50) UNIQUE NOT NULL DEFAULT '',
  contact text NOT NULL DEFAULT '',

  PRIMARY KEY(id)
) INHERITS(ObjetReseau);

/**
  * Proprietaire, peut possder des lignes, des connecteurs, des cabines ou des cbles.
  */
CREATE TABLE Proprietaire (
  idProprietaire serial NOT NULL,

  numAbonnement varchar(50) NOT NULL DEFAULT 0,

  PRIMARY KEY(idProprietaire)
) INHERITS (Personne);

/**
  * Une super cabine est regroupement de plusieurs cabines.
  */
CREATE TABLE SuperCabine (
  id serial NOT NULL,

  nom varchar(100) NOT NULL,
  numeroAbonne integer NOT NULL DEFAULT 0,
  acces text NOT NULL DEFAULT '',
  etat integer NOT NULL DEFAULT 0,

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);

/**
  * Reprsente une commune.
  */
CREATE TABLE Commune (
  id serial NOT NULL,
  nom varchar(100) UNIQUE NOT NULL,

  PRIMARY KEY(id)
) INHERITS(ObjetReseau);

/**
  * Une cabine est un emplacement gographique dans lequel
  * les fibres optiques sont interconnectes.
  */
CREATE TABLE Cabine (
  id serial NOT NULL,
  superCabine integer NULL REFERENCES SuperCabine,
  commune integer NULL REFERENCES Commune,
  proprietaire integer NULL REFERENCES Proprietaire,

  nom varchar(100) NOT NULL,
  numeroAbonne integer NOT NULL DEFAULT 0,
  acces text NOT NULL DEFAULT '',
  etat integer NOT NULL DEFAULT 0,

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);
CREATE INDEX Cabine_superCabine_INDEX on Cabine(superCabine);
CREATE INDEX Cabine_commune_INDEX on Cabine(commune);
CREATE INDEX Cabine_proprietaire_INDEX on Cabine(proprietaire);

/**
  * Reprsent un groupe de ports qui est associ  un connecteur.
  */
CREATE TABLE GroupePorts (
  id serial NOT NULL,

  PRIMARY KEY(id)
);

/**
  * Un fabriquant, utilis uniquement pour les cbles et bretelles.
  */
CREATE TABLE Fabriquant (
  id serial NOT NULL,

  nom varchar(100) NOT NULL,
  description text NOT NULL DEFAULT '',
  
  PRIMARY KEY(id)
) INHERITS(ObjetReseau);

/**
  * Un connecteur est un lment permettant de connecter les fibres
  * optique entre elle  l'intrieur d'une cabine.
  */
CREATE TABLE Connecteur (
  id serial NOT NULL,

  --  qui appartient le connecteur ?
  proprietaire integer NULL REFERENCES Proprietaire,
  -- la cabine dans laquelle se trouve le connecteur
  cabine integer NOT NULL REFERENCES Cabine ON DELETE CASCADE,
  -- les ports qu'il possde
  groupePorts integer NULL REFERENCES GroupePorts,
  -- le fabriquant du connexteur
  fabriquant integer NULL REFERENCES Fabriquant,


  nom varchar(255) NOT NULL DEFAULT '',
  etat integer NOT NULL DEFAULT 0,
  datePose date NULL,
  cout integer NOT NULL DEFAULT 0,
  taille integer NOT NULL DEFAULT 0,

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);
CREATE INDEX Connecteur_proprietaire_INDEX on Connecteur(proprietaire);
CREATE INDEX Connecteur_cabine_INDEX on Connecteur(cabine);
CREATE INDEX Connecteur_groupePorts_INDEX on Connecteur(groupePorts);
CREATE INDEX Connecteur_fabriquant_INDEX on Connecteur(fabriquant);

/**
  * Un port est un lment de connecteur permettant d'y connecter
  * des fibres optiques.
  */
CREATE TABLE Port (
  id serial NOT NULL,
  -- clef trangre vers un autre port (lorsque des ports sont lies ensembles)
  port integer NULL REFERENCES Port,
  -- clef trangre,  quel groupe de ports appartient le port ?
  groupePorts integer NOT NULL REFERENCES GroupePorts ON DELETE CASCADE,

  -- numro du port 
  numero integer NULL,
  -- cot sur lequel il se trouve
  cote integer NULL,

  PRIMARY KEY(id)
);
CREATE INDEX Port_port_INDEX on Port(port);
CREATE INDEX Port_groupePorts_INDEX on Port(groupePorts);
-- voir le document 'contraintes d'intgrit'
CREATE TRIGGER trigger_port
BEFORE INSERT OR UPDATE
ON Port FOR EACH ROW
EXECUTE PROCEDURE trigger_modification_ou_ajout_port();
CREATE TRIGGER trigger_port2
AFTER INSERT OR UPDATE
ON Port FOR EACH ROW
EXECUTE PROCEDURE trigger_modification_ou_ajout_port_apres();

/**
  * Un adaptateur se place sur les ports des distributeurs
  * et permette d'y connecter des bretelles.
  */
CREATE TABLE Adaptateur (
  id serial NOT NULL,
  
  -- le type de connecteur
  typeConnecteur integer NOT NULL REFERENCES TypeConnecteur ON DELETE CASCADE,
  -- le port sur lequel est plac l'adaptateur
  port integer NOT NULL REFERENCES Port ON DELETE CASCADE,

  PRIMARY KEY(id)
);
CREATE INDEX Adaptateur_typeConnecteur_INDEX on Adaptateur(typeConnecteur);
CREATE INDEX Adaptateur_port_INDEX on Adaptateur(port);

/**
  * Reprsente un ensemble de fibre optique formant un parcours d'un
  * point  un autre
  */
CREATE TABLE Ligne (
  id serial NOT NULL,

  --un personne possdant la ligne
  proprietaire integer NOT NULL REFERENCES Proprietaire ON DELETE CASCADE,

  nom varchar(100) NOT NULL,
  couleur integer NOT NULL DEFAULT 0,
  numContrat varchar(100) NOT NULL DEFAULT '',
  dateActivation date NULL,
  longueur integer NOT NULL DEFAULT 0,

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);
CREATE INDEX Ligne_proprietaire_INDEX on Ligne(proprietaire);

/**
  * Reprsente une attenuation (dB) en fonction du diamtre (pour les lignes). 
  */
CREATE TABLE Attenuation (
  id serial NOT NULL,

  ligne integer NOT NULL REFERENCES Ligne ON DELETE CASCADE,

  diametre integer NOT NULL,
  attenuation integer NOT NULL,

  PRIMARY KEY(id)
);
CREATE INDEX Attenuation_ligne_INDEX on Attenuation(ligne);

/**
  * Reprsente n'importe quelle fibre optique :  l'intrieur des cbles entre
  * les cabines ou  l'intrieur des cabines sous forme de bretelles.
  */
CREATE TABLE LiaisonOptique (
  id serial NOT NULL,

  port1 integer NULL REFERENCES Port,
  port2 integer NULL REFERENCES Port,
  ligne integer NULL REFERENCES Ligne,

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);
CREATE INDEX LiaisonOptique_port1_INDEX on LiaisonOptique(port1);
CREATE INDEX LiaisonOptique_port2_INDEX on LiaisonOptique(port2);
CREATE INDEX LiaisonOptique_ligne_INDEX on LiaisonOptique(ligne);

/**
  * Permet de grouper des bretelles entre elle
  */
CREATE TABLE GroupeBretelles (
  id serial NOT NULL,
  
  PRIMARY KEY(id)
);

/**
  * Un fournisseur, utilis uniquement pour les bretelles actuellement.
  */
CREATE TABLE Fournisseur (
  id serial NOT NULL,

  nom varchar(100) NOT NULL,
  
  PRIMARY KEY(id)
) INHERITS (ObjetReseau);

/**
  * Une couleur, utilise uniquement pour les bretelles actuellement.
  */
CREATE TABLE Couleur (
  id serial NOT NULL,

  nom varchar(100) NOT NULL,
  valeur integer NULL,

  PRIMARY KEY(id)
) INHERITS (ObjetFibernet);

/**
  * Le type d'une bretelle.
  */
CREATE TABLE TypeBretelle (
  idTypeBretelle serial NOT NULL,

  PRIMARY KEY(idTypeBretelle)
) INHERITS("Type");

/**
  * Une fibre optique seule permettant de connecter des connecteur
  *  l'intrieur d'une cabine
  * Pour qu'une bretelle puisse tre connecte  un port d'un distributeur il faut
  * que le distributeur possde un adaptateur sur le port en question du mme type que celui
  * de la bretelle.
  * Une bretelle appartient  une cabine.
  */
CREATE TABLE Bretelle (
  id_Bretelle serial NOT NULL,
  typeConnecteur integer NOT NULL REFERENCES TypeConnecteur ON DELETE CASCADE,
  typeBretelle integer NOT NULL REFERENCES typeBretelle ON DELETE CASCADE,
  cabine integer NOT NULL REFERENCES Cabine ON DELETE CASCADE,
  groupeBretelles integer NULL REFERENCES GroupeBretelles,
  fournisseur integer NULL REFERENCES Fournisseur,
  fabriquant integer NULL REFERENCES Fabriquant,
  couleur integer NULL REFERENCES Couleur,

  modele integer NOT NULL DEFAULT 0, -- 0 = SIMPLEX, 1 = DUPLEX
  longueur integer NOT NULL DEFAULT 0,
  utilisation text NOT NULL DEFAULT '',
  etat integer NOT NULL DEFAULT 0, -- 0 = EXISTANT, 1 = PREVU
  datePose date NULL,

  PRIMARY KEY(id_Bretelle)
) INHERITS(LiaisonOptique);
CREATE INDEX Bretelle_typeConnecteur_INDEX on Bretelle(typeConnecteur);
CREATE INDEX Bretelle_typeBretelle_INDEX on Bretelle(typeBretelle);
CREATE INDEX Bretelle_cabine_INDEX on Bretelle(cabine);
CREATE INDEX Bretelle_groupeBretelles_INDEX on Bretelle(groupeBretelles);
CREATE INDEX Bretelle_fournisseur_INDEX on Bretelle(fournisseur);
CREATE INDEX Bretelle_fabriquant_INDEX on Bretelle(fabriquant);
CREATE INDEX Bretelle_couleur_INDEX on Bretelle(couleur);
-- voir le document 'contraintes d'intgrit'
CREATE TRIGGER trigger_bretelle
BEFORE INSERT OR UPDATE
ON Bretelle FOR EACH ROW
EXECUTE PROCEDURE trigger_modification_ou_ajout_bretelle();

/**
  * Ensemble de fibre optique liant des cabines entre elles.
  */
CREATE TABLE Cable (
  id serial NOT NULL,

  fabriquant integer NULL REFERENCES Fabriquant,

  -- une personne peut possder un cble
  proprietaire integer NULL REFERENCES Proprietaire,

  etat integer NOT NULL DEFAULT 0,
  datePose date NULL,
  coutUnite integer NOT NULL DEFAULT 0,
  longueur integer NOT NULL DEFAULT 0,
  description text NOT NULL DEFAULT '',
  numeroCommande varchar(50) NOT NULL DEFAULT '',

  PRIMARY KEY(id)
) INHERITS (ObjetReseau);
CREATE INDEX Cable_fabriquant_INDEX on Cable(fabriquant);
CREATE INDEX Cable_proprietaire_INDEX on Cable(proprietaire);

/**
  * Un sous-cable est un segment de cable reliant 2 cabines.
  */
CREATE TABLE SousCable (
  id serial NOT NULL,

  -- les deux cabines que relie le sous-cble
  cabine1 integer NOT NULL REFERENCES Cabine ON DELETE CASCADE, 
  cabine2 integer NOT NULL REFERENCES Cabine ON DELETE CASCADE,

  -- le cables auquel appartient le sous-cable
  cable integer NOT NULL REFERENCES Cable ON DELETE CASCADE,

  PRIMARY KEY(id)
) INHERITS (ObjetFibernet);
CREATE INDEX SousCable_cabine1_INDEX on SousCable(cabine1);
CREATE INDEX SousCable_cabine2_INDEX on SousCable(cabine2);
CREATE INDEX SousCable_cable_INDEX on SousCable(cable);

/**
  * Le type d'une cassette
  */
CREATE TABLE TypeCassette (
  idTypeCassette serial NOT NULL,

  PRIMARY KEY(idTypeCassette)
) INHERITS("Type");


/**
  * Reprsente une boite qui permet de relier ensemble diffrentes
  * fibres optiques  l'intrieur d'une cabine.
  * Possde un certain type de connecteur pour l'ensemble de ses ports.
  */
CREATE TABLE Cassette (

  typeConnecteur integer NOT NULL REFERENCES TypeConnecteur ON DELETE CASCADE,
  typeCassette integer NOT NULL REFERENCES TypeCassette ON DELETE CASCADE,

  attenuation float NOT NULL DEFAULT 0.0,
  position varchar(20) NOT NULL DEFAULT ''
) INHERITS(Connecteur);
CREATE INDEX Cassette_typeConnecteur_INDEX on Cassette(typeConnecteur);
CREATE INDEX Cassette_TypeCassette_INDEX on Cassette(TypeCassette);

/**
  * Les type d'un distributeur, par exemple : 
  */
CREATE TABLE TypeDistributeur (
  idTypeDistributeur serial NOT NULL,

  PRIMARY KEY(idTypeDistributeur)
) INHERITS("Type");

/**
  * Permet de relier les fibres optiques internes aux cables entrants
  * d'une cabine.
  */
CREATE TABLE Distributeur (
   typeDistributeur integer NOT NULL REFERENCES TypeDistributeur ON DELETE CASCADE,

   position varchar(20) NOT NULL DEFAULT ''
) INHERITS(Connecteur);
CREATE INDEX Terminal_typeDistributeur_INDEX on Distributeur(typeDistributeur);

/**
  * Les paramtres globaux du projet.
  * Il n'y a qu'une seule instance
  */
CREATE TABLE ParametresProjet (
  id serial NOT NULL,

  nbFibreTubeDefaut integer NOT NULL,

  PRIMARY KEY(id)
);

/**
  * Permet de lier plusieurs bretelles ensembles  l'intrieur d'une cabine.
  */
CREATE TABLE Soudure (
  numero varchar(20) NOT NULL DEFAULT ''
) INHERITS(Connecteur);

/**
  * Les type d'un terminal, par exemple : 
  */
CREATE TABLE TypeTerminal (
  idTypeTerminal serial NOT NULL,

  PRIMARY KEY(idTypeTerminal)
) INHERITS("Type");

/**
  * Element terminal dans une cabine.
  */
CREATE TABLE Terminal (
   typeTerminal integer NULL REFERENCES TypeTerminal ON DELETE CASCADE
) INHERITS(Connecteur);
CREATE INDEX Terminal_typeTerminal_INDEX on Terminal(TypeTerminal);

/**
  * Les types de tubes.
  */
CREATE TABLE TypeTube (
  idTypeTube serial NOT NULL,

  PRIMARY KEY(idTypeTube)
) INHERITS("Type");

/**
  * Se trouve  l'intrieur des cbles, possde un certain nombre de fibre optique.
  */
CREATE TABLE Tube (
  id serial NOT NULL,

  typeTube integer NULL REFERENCES TypeTube,
  sousCable integer NOT NULL REFERENCES SousCable ON DELETE CASCADE,

  PRIMARY KEY(id)
) INHERITS (ObjetFibernet);
CREATE INDEX Tube_typeTube_INDEX on Tube(typeTube);
CREATE INDEX Tube_sousCable_INDEX on Tube(sousCable);

/**
  * Fibre optique se trouvant dans les cbles (entre les cabines).
  */
CREATE TABLE FibreOptiqueExterne (
  tube integer NOT NULL REFERENCES Tube ON DELETE CASCADE 
) INHERITS(LiaisonOptique);
CREATE INDEX FibreOptiqueExterne_tube_INDEX on FibreOptiqueExterne(tube);

